/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: StatsDlg.cpp
**
** DESCRIPTION: This program implements the methods of CStatsDlg class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang       March-2001
**
/**************************************************************************/


// StatsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "RealTracker.h"
#include "StatsDlg.h"

#include "RealTrackerView.h"
#include "VQTest.h"
#include "MediaStatistics.h"
#include "FtpSave.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStatsDlg dialog


CStatsDlg::CStatsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CStatsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CStatsDlg)
	//}}AFX_DATA_INIT
    m_EditFont.m_hObject = NULL;
}


void CStatsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CStatsDlg)
	DDX_Control(pDX, IDC_EDIT_SHOWSTATS, m_ShowStats);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CStatsDlg, CDialog)
	//{{AFX_MSG_MAP(CStatsDlg)
	ON_BN_CLICKED(IDC_BUTTON_SAVE, OnButtonSave)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStatsDlg message handlers

BOOL CStatsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    char CurDir[256];
    GetCurrentDirectory(256, CurDir);
    m_CurSaveDir = CurDir;
	
	return TRUE;  // return TRUE unless you set the focus to a control
}

void CStatsDlg::SetTopView(CRealTrackerView *pView)
{
    m_pView = pView;
}

void CStatsDlg::ShowStatsData(char *str)
{
    if (m_EditFont.m_hObject == NULL)
    {
	    LOGFONT logFont;
	    memset(&logFont, 0, sizeof(logFont));
        logFont.lfHeight = 8;
	    logFont.lfPitchAndFamily = VARIABLE_PITCH | FF_SWISS;
	    lstrcpy(logFont.lfFaceName, "Courier");
        m_EditFont.CreateFontIndirect(&logFont);
        GetDlgItem(IDC_EDIT_SHOWSTATS)->SetFont(&m_EditFont);
    }
    SetDlgItemText(IDC_EDIT_SHOWSTATS, str);
}

void CStatsDlg::OnOK() 
{
    m_pView->HideStatsDlg();
    ShowWindow(SW_HIDE);
}

void CStatsDlg::OnClose() 
{
    m_pView->HideStatsDlg();
	CDialog::OnClose();
}

void CStatsDlg::OnButtonSave() 
{
//    SetCurrentDirectory(m_CurSaveDir);
    CFileDialog fDlg(FALSE, "log", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "log files (*.log)|*.log|all files (*.*)|*.*||", this);
    fDlg.m_ofn.lpstrInitialDir = m_CurSaveDir;
    if (fDlg.DoModal() == IDOK)
    {
        CString fname = fDlg.GetPathName();
        FILE *fp = fopen(fname, "w");
        if (fp == NULL)
            return;
        CMediaStatistics *pVideoStats = m_pView->GetVQTest()->GetMediaLog()->GetVideoStats();
        VQDataLink *pLink;
        VQData *pData = pVideoStats->GetVQDataFirst(pLink);
        fprintf(fp, "CodedBw AvgBw TotalBw CurBw AvgLtcy Transport CodedFRate AvgFRate CurFRate FDrop FLost AvgJit CurJit MaxJit CpuUsage\n");
        while (pLink)
        {
            fprintf(fp, "%d %d %d %d %d %d %.2lf %.2lf %.2lf %d %d %.2lf %.2lf %.2lf %.2lf\n", 
                        pData->CodedBandWidth,
                        pData->AverageBandWidth,
                        pData->TotalBandWidth,
                        pData->CurrentBandWidth,
                        pData->AverageLatency,
                        pData->TransportMode,
                        pData->CodedFrameRate,
                        pData->AverageFrameRate,
                        pData->CurrentFrameRate,
                        pData->FramesDropped,
                        pData->FramesLost,
                        pData->AverageJitter,
                        pData->CurrentJitter,
                        pData->MaxJitter,
                        pData->CpuUsage);
            pData = pVideoStats->GetVQDataNext(pLink);
        }
        fprintf(fp, "CodedBw AvgBw TotalBw CurBw AvgLtcy Transport CodedFRate AvgFRate CurFRate FDrop FLost AvgJit CurJit MaxJit CpuUsage\n");
        fclose(fp);

        /*CString fname1 = fname.Left(fname.GetLength() - 4) + "-1.log";
        fp = fopen(fname1, "w");
        if (fp == NULL)
            return;
        CMediaLog *pMediaLog = m_pView->GetVQTest()->GetMediaLog();
        /*int i = 0, j, k1, k2;
        BOOL contWrite = TRUE;
        while ( contWrite )
        {
            contWrite = FALSE;
            k1 = i;
            k2 = pMediaLog->m_NumPerSec[0];
            for (j=0; j<pMediaLog->m_NumSec; j++)
            {
                if (k1 >= k2)
                    fprintf(fp, " \t");
                else
                {
                    fprintf(fp, "%.2lf\t", pMediaLog->m_FrameIntervalBuf[k1]);
                    contWrite = TRUE;
                }
                k1 += pMediaLog->m_NumPerSec[j];
                k2 += pMediaLog->m_NumPerSec[j+1];
            }
            fprintf(fp, "\n");
            i ++;
        }* /

        int i, j, k;
        k = 0;
        for (i=0; i<pMediaLog->m_NumSec; i++)
        {
            if (pMediaLog->m_NumPerSec[i] == 0)
            {
                fprintf(fp, "XXX\n");
                continue;
            }
            for (j=0; j<pMediaLog->m_NumPerSec[i]; j++)
                fprintf(fp, "%.2lf\t", pMediaLog->m_FrameIntervalBuf[k++]);
            fprintf(fp, "\n");
        }

        fclose(fp);*/

        char CurDir[256];
        GetCurrentDirectory(256, CurDir);
        m_CurSaveDir = CurDir;
    }
}
