/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: VQTest.cpp
**
** DESCRIPTION: This program implements the methods of CVQTest class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang       March-2001
**
/**************************************************************************/


// VQTest.cpp: implementation of the CVQTest class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RealTracker.h"
#include "VQTest.h"
#include "StatsDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVQTest::CVQTest(CRealTrackerView *pView)
{
    m_pView = pView;
    m_MediaLog = new CMediaLog(this);
    m_MediaLog->InitVideoStats(600);
    m_RMAContext = new CRMAContext(this);
}

CVQTest::~CVQTest()
{
    if (m_MediaLog != NULL)
        delete [] m_MediaLog;
    if (m_RMAContext != NULL)
        delete m_RMAContext;
}

CRealTrackerView *CVQTest::GetRealTrackerView()     { return m_pView; }
CMediaLog *CVQTest::GetMediaLog()                   { return m_MediaLog; }
CRMAContext *CVQTest::GetRMAContext()               { return m_RMAContext; }

void CVQTest::ShowTimeLine(long CurTime, long TotalTime)
{
    int Progress;
    int CurTime_MSec, CurTime_Sec, CurTime_Min, CurTime_Hur;
    int TotalTime_MSec, TotalTime_Sec, TotalTime_Min, TotalTime_Hur;
    char str[1024];

    Progress = CurTime * 100 / TotalTime;

    CurTime_MSec = CurTime % 1000;
    CurTime_Sec = CurTime / 1000;
    CurTime_Min = CurTime_Sec / 60;
    CurTime_Sec = CurTime_Sec % 60;
    CurTime_Hur = CurTime_Min / 60;
    CurTime_Min = CurTime_Min % 60;

    TotalTime_MSec = TotalTime % 1000;
    TotalTime_Sec = TotalTime / 1000;
    TotalTime_Min = TotalTime_Sec / 60;
    TotalTime_Sec = TotalTime_Sec % 60;
    TotalTime_Hur = TotalTime_Min / 60;
    TotalTime_Min = TotalTime_Min % 60;

    sprintf(str, "%02d:%02d:%02d.%03d/%02d:%02d:%02d.%03d",
                  CurTime_Hur, CurTime_Min, CurTime_Sec, CurTime_MSec,
                  TotalTime_Hur, TotalTime_Min, TotalTime_Sec, TotalTime_MSec);

    m_pView->ShowProgressBarAndTimeLine(Progress, str);
}

void CVQTest::InitAudioStats(int maxDataNum)
{
    m_MediaLog->InitAudioStats(maxDataNum);
}

void CVQTest::InitVideoStats(int maxDataNum)
{
    m_MediaLog->InitVideoStats(maxDataNum);
}

void CVQTest::AddAudioStats(VQData *pData)
{
    m_MediaLog->AddAudioStatsData(pData);
}

void CVQTest::AddVideoStats(VQData *pData)
{
    static char TransportModeName[][10] =
    {
        "XXXX", "TCP", "UDP", "RTSPvHTTP", "HTTP", "Unknown"
    };

    static char VQDataNames[][20] = 
    {
        "ServerInfo",
        "CodecName",
        "Name",
        "CodedBandWidth",
        "AverageBandWidth",
        "TotalBandWidth",
        "CurrentBandWidth",
        "AverageLatency",
        "TransportProtocol",
        "CodedFrameRate",
        "AverageFrameRate",
        "CurrentFrameRate",
        "FramesDropped",
        "FramesLost",
        "AverageJitter",
        "CurrentJitter",
        "MaxJitter",
        "CpuUsage"
    };

    CStatsDlg *statsDlg = m_pView->GetStatsDlg();
    if (statsDlg != NULL && m_pView->StatsDlgIsVisable())
    {
        char strBuf[4096], str[256];
        int i;

        i = 0;
        sprintf(str, "%-20s%s\r\n", VQDataNames[i++], (pData->ServerInfo != NULL ? pData->ServerInfo : "Unknown"));
        strcpy(strBuf, str);
        sprintf(str, "%-20s%s\r\n", VQDataNames[i++], (pData->VideoCodecName != NULL ? pData->VideoCodecName : "Unknown"));
        strcat(strBuf, str);
        sprintf(str, "%-20s%s\r\n", VQDataNames[i++], pData->Name);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->CodedBandWidth);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->AverageBandWidth);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->TotalBandWidth);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->CurrentBandWidth);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->AverageLatency);
        strcat(strBuf, str);
        sprintf(str, "%-20s%s\r\n", VQDataNames[i++], TransportModeName[pData->TransportMode]);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->CodedFrameRate);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->AverageFrameRate);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->CurrentFrameRate);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->FramesDropped);
        strcat(strBuf, str);
        sprintf(str, "%-20s%d\r\n", VQDataNames[i++], pData->FramesLost);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->AverageJitter);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->CurrentJitter);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->MaxJitter);
        strcat(strBuf, str);
        sprintf(str, "%-20s%.2lf\r\n", VQDataNames[i++], pData->CpuUsage);
        strcat(strBuf, str);

        //statsDlg->SetDlgItemText(IDC_EDIT_SHOWSTATS, strBuf);
        statsDlg->ShowStatsData(strBuf);
    }

    m_MediaLog->AddVideoStatsData(pData);
}

void CVQTest::AddVideoJitter(VQJitter *pData)
{
    m_MediaLog->AddVideoJitterData(pData);
}

