/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: RealDataDoc.cpp
**
** DESCRIPTION: This program implements the methods of CRealDataDoc class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang        March-2001
**
/**************************************************************************/


// RealDataDoc.cpp : implementation of the CRealDataDoc class
//

#include "stdafx.h"
#include "RealData.h"

#include "RealDataDoc.h"
#include "RealDataView.h"
#include "RealTreeView.h"
#include "RealListView.h"
#include "NewFolderDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRealDataDoc

IMPLEMENT_DYNCREATE(CRealDataDoc, CDocument)

BEGIN_MESSAGE_MAP(CRealDataDoc, CDocument)
	//{{AFX_MSG_MAP(CRealDataDoc)
	ON_COMMAND(ID_FROMEMAIL, OnFromemail)
	ON_COMMAND(ID_DATA_NEWFOLDER, OnDataNewfolder)
	ON_COMMAND(ID_DATA_ADDFILES, OnDataAddfiles)
	ON_COMMAND(ID_CDF_FR_ALL, OnCdfFrAll)
	ON_COMMAND(ID_CDF_FR_SELECTED, OnCdfFrSelected)
	ON_COMMAND(ID_CDF_BW_ALL, OnCdfBwAll)
	ON_COMMAND(ID_CDF_BW_SELECTED, OnCdfBwSelected)
	ON_COMMAND(ID_CDF_JITTER_ALL, OnCdfJitterAll)
	ON_COMMAND(ID_CDF_JITTER_SELECTED, OnCdfJitterSelected)
	ON_COMMAND(ID_COUNT, OnCount)
	ON_COMMAND(ID_EXPORT, OnExport)
	ON_COMMAND(ID_EDIT_DELETE, OnEditDelete)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRealDataDoc construction/destruction

CRealDataDoc::CRealDataDoc()
{
    m_pRealTraceData = NULL;
}

CRealDataDoc::~CRealDataDoc()
{
    if (m_pRealTraceData != NULL)
        delete m_pRealTraceData;
}

BOOL CRealDataDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

    if (m_pRealTraceData != NULL)
        delete m_pRealTraceData;

    m_pRealTraceData = new CRealTraceData();
    UpdateAllViews(NULL);

    SetModifiedFlag(FALSE);

	return TRUE;
}

BOOL CRealDataDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
//	if (!CDocument::OnOpenDocument(lpszPathName))
//		return FALSE;

    FILE *fp = fopen(lpszPathName, "r");
    if (fp == NULL)
    {
        CString msg;
        msg.Format("Can't open file %s", lpszPathName);
        AfxGetMainWnd()->MessageBox(msg, "Error");
        return FALSE;
    }

    char buffer[4096];

    CRealTraceData *newData = new CRealTraceData();
    fgets(buffer, 4096, fp);
    buffer[strlen(buffer)-1] = 0;
    if (strcmp(buffer, "[FOLDER NUMBER]") != 0)
    {
        AfxGetMainWnd()->MessageBox("Format error", "Error");
        delete newData;
        return FALSE;
    }

    fgets(buffer, 4096, fp);
    buffer[strlen(buffer)-1] = 0;
    int itemNum = atoi(buffer);
    if (itemNum < 0 || !isdigit(buffer[0]))
    {
        AfxGetMainWnd()->MessageBox("Format error", "Error");
        delete newData;
        return FALSE;
    }

    int realItemNum = 0;

    if (! feof(fp))
    {
        fgets(buffer, 4096, fp);
        buffer[strlen(buffer)-1] = 0;
    }

    while (! feof(fp) )
    {
        if (buffer[0] != '[' || buffer[strlen(buffer)-1] != ']')
        {
            AfxGetMainWnd()->MessageBox("Format error", "Error");
            delete newData;
            return FALSE;
        }

        buffer[strlen(buffer)-1] = 0;
        newData->AddItem(buffer+1);
        realItemNum ++;

        while (! feof(fp) )
        {
            if (fgets(buffer, 4096, fp) == NULL)                        // end of file
                break;
            buffer[strlen(buffer)-1] = 0;
            if (buffer[0] == '[' || buffer[strlen(buffer)-1] == ']')    // next folder
                break;
            newData->AddSubItem(realItemNum-1, buffer);
        }
    }

    if (itemNum != realItemNum)
    {
        AfxGetMainWnd()->MessageBox("Format error", "Error");
        delete newData;
        return FALSE;
    }

    if (m_pRealTraceData != NULL)
        delete m_pRealTraceData;

    m_pRealTraceData = newData;
	
    SetModifiedFlag(FALSE);

    return TRUE;
}

BOOL CRealDataDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
    FILE *fp = fopen(lpszPathName, "w");
    if (fp == NULL)
    {
        CString msg;
        msg.Format("Can't save to %s", lpszPathName);
        AfxGetMainWnd()->MessageBox(msg, "Error");
        return FALSE;
    }

    int itemNum, subItemNum, i, j;

    itemNum = m_pRealTraceData->GetItemNum();
    fprintf(fp, "[FOLDER NUMBER]\n%d\n", itemNum);
    for (i=0; i<itemNum; i++)
    {
        fprintf(fp, "[%s]\n", (LPCTSTR)m_pRealTraceData->GetItemName(i));
        subItemNum = m_pRealTraceData->GetSubItemNum(i);
        for (j=0; j<subItemNum; j++)
            fprintf(fp, "%s\n", (LPCTSTR)m_pRealTraceData->GetSubItemPath(i, j));
    }

    fclose(fp);

    SetModifiedFlag(FALSE);

    return TRUE;

//	return CDocument::OnSaveDocument(lpszPathName);
}



/////////////////////////////////////////////////////////////////////////////
// CRealDataDoc serialization

void CRealDataDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CRealDataDoc diagnostics

#ifdef _DEBUG
void CRealDataDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CRealDataDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRealDataDoc commands

CRealTraceData *CRealDataDoc::GetRealTraceData()
{
    return m_pRealTraceData;
}

void CRealDataDoc::OnFromemail() 
{
/*    CFileDialog fDlg(TRUE, NULL, NULL, OFN_HIDEREADONLY, "all files (*.*)|*.*||", AfxGetApp()->GetMainWnd());
//    fDlg.m_ofn.lpstrInitialDir = m_CurBrowserDir;
    if (fDlg.DoModal() != IDOK)
        return;

    CString file = fDlg.GetPathName();
    FILE *fp1, *fp2;
    fp1 = fopen(file, "r");
    if (fp1 == NULL)
        return;

    char sym = '\"';
    char buf[256], *subject, *body, *end;
    CString logfile;

    BOOL bFirstLog = TRUE;
    fgets(buf, 256, fp1);  // skip the first line: "Subject","Body"
    while (fgets(buf, 256, fp1))
    {
        if (buf[0] != sym)
            continue;
        if (strlen(buf) > 0 && buf[strlen(buf)-1] == 10)
            buf[strlen(buf)-1] = 0;
        subject = buf + 1;
        body = strchr(subject, sym);
        *body = 0;
        body += 3;    // skip ","
        logfile = m_pRealTraceData->GetRootDir() + "\\" + GetLogFileName(subject, bFirstLog);
        fp2 = fopen(logfile, "w");
        fprintf(fp2, "%s\n", body);
        while (fgets(buf, 256, fp1))
        {
            if (strlen(buf) > 0 && buf[strlen(buf)-1] == 10)
                buf[strlen(buf)-1] = 0;
            end = strchr(buf, sym);
            if (end == NULL)
                fprintf(fp2, "%s\n", buf);
            else
            {
                if (buf != end)
                {
                    *end = 0;
                    fprintf(fp2, "%s", buf);
                }
                fclose(fp2);
                break;
            }
        }
    }
    fclose(fp1);
*/
}

// RealTracer C260601-A 00294211300888935520-040
// RealTracer C260601-A 00294211300509935520-039 - FILE NOT FOUND !!!
CString CRealDataDoc::GetLogFileName(char *subject, BOOL &bFirstLog)
{
/*    char *dirname, *filename, *end;
    char space = ' ';

    dirname = strchr(subject, space) + 1;
    filename = strchr(dirname, space);
    *filename = 0;
    filename ++;
    BOOL FILE_NOT_FOUND = (strstr(filename, "FILE NOT FOUND") != NULL);
    end = strchr(filename, space);
    if (end != NULL)
        *end = 0;
    end = strchr(filename, '(');  // ingnore the ip address
    if (end != NULL)
        *end = 0;
    if (FILE_NOT_FOUND)
        strcat(filename, "__FILE_NOT_FOUND");

    if (bFirstLog)
    {
        bFirstLog = FALSE;
        CreateDirectory(m_pRealTraceData->GetRootDir() + "\\" + dirname, NULL);
    }

    return CString(dirname) + "\\" + filename;
*/
    return "";
}

CRealTreeView *CRealDataDoc::GetTreeView()
{
    CView *pView = NULL;
    POSITION pos = GetFirstViewPosition();
    while (pos != NULL)
    {
        pView = GetNextView(pos);
        if (pView->IsKindOf(RUNTIME_CLASS(CRealDataView)))
            break;
        pView = NULL;
    }
    if (pView == NULL)
        return NULL;
    else
        return ((CRealDataView *)pView)->GetTreeView();
}

CRealListView *CRealDataDoc::GetListView()
{
    CView *pView = NULL;
    POSITION pos = GetFirstViewPosition();
    while (pos != NULL)
    {
        pView = GetNextView(pos);
        if (pView->IsKindOf(RUNTIME_CLASS(CRealDataView)))
            break;
        pView = NULL;
    }
    if (pView == NULL)
        return NULL;
    else
        return ((CRealDataView *)pView)->GetListView();
}

void CRealDataDoc::OnDataNewfolder() 
{
    CNewFolderDlg dlg;

    if (dlg.DoModal() != IDOK)
        return;

    m_pRealTraceData->AddItem(dlg.m_FolderName);
    GetTreeView()->AddItem(m_pRealTraceData->GetItemNum() - 1);

    SetModifiedFlag();
}

void CRealDataDoc::OnDataAddfiles() 
{
    CRealTreeView *pTreeView = GetTreeView();
    if (pTreeView->m_hPreviousSelectedItem == NULL)
    {
        AfxGetApp()->GetMainWnd()->MessageBox("Select one folder to add files", "Error");
        return;
    }

    CTreeCtrl *pTreeCtrl = &(pTreeView->GetTreeCtrl());
    int index = pTreeCtrl->GetItemData(pTreeView->m_hPreviousSelectedItem);
    if (index > m_pRealTraceData->GetItemNum())
    {
        AfxGetApp()->GetMainWnd()->MessageBox("Select one folder to add files", "Error");
        return;
    }

    CFileDialog fDlg(TRUE, NULL, NULL, OFN_HIDEREADONLY, "all files (*.*)|*.*||", AfxGetApp()->GetMainWnd());
    fDlg.m_ofn.Flags |= OFN_ALLOWMULTISELECT;
    fDlg.m_ofn.lpstrFile = new char [16384];  // 16K
    fDlg.m_ofn.lpstrFile[0] = 0;
    fDlg.m_ofn.nMaxFile = 16384;

//    if (fDlg.DoModal() != IDOK)
    int status = fDlg.DoModal();
    if (status == IDCANCEL)
    {
        delete [] fDlg.m_ofn.lpstrFile;
        return;
    }

    CRealListView *pListView = GetListView();

    POSITION pos = fDlg.GetStartPosition();
    CString filePath;
    int num = m_pRealTraceData->GetSubItemNum(index);
    while (pos)
    {
        filePath = fDlg.GetNextPathName(pos);
        m_pRealTraceData->AddSubItem(index, filePath);
        pListView->AddToListView(m_pRealTraceData->GetSubItemData(index, num));
        num ++;
    }

    delete [] fDlg.m_ofn.lpstrFile;

    SetModifiedFlag();
}

void CRealDataDoc::OnCdfFrAll() 
{
	// TODO: Add your command handler code here
	GetListView()->OnCdfFrAll();
}

void CRealDataDoc::OnCdfFrSelected() 
{
	// TODO: Add your command handler code here
	GetListView()->OnCdfFrSelected();
}

void CRealDataDoc::OnCdfBwAll() 
{
	// TODO: Add your command handler code here
	GetListView()->OnCdfBwAll();
}

void CRealDataDoc::OnCdfBwSelected() 
{
	// TODO: Add your command handler code here
	GetListView()->OnCdfBwSelected();
}

void CRealDataDoc::OnCdfJitterAll() 
{
	// TODO: Add your command handler code here
	GetListView()->OnCdfJitterAll();
}

void CRealDataDoc::OnCdfJitterSelected() 
{
	// TODO: Add your command handler code here
    GetListView()->OnCdfJitterSelected();	
}

void CRealDataDoc::OnCount() 
{
	// TODO: Add your command handler code here
	GetListView()->OnCount();
}

void CRealDataDoc::OnExport() 
{
	// TODO: Add your command handler code here
	GetListView()->OnExport();
}

void CRealDataDoc::OnEditDelete() 
{
	// TODO: Add your command handler code here
	GetListView()->OnEditDelete();
    SetModifiedFlag();
}
