/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: RealDataView.cpp
**
** DESCRIPTION: This program implements the methods of CRealDataView class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang        March-2001
**
/**************************************************************************/


// RealDataView.cpp : implementation of the CRealDataView class
//

#include "stdafx.h"
#include "RealData.h"

#include "RealDataDoc.h"
#include "RealDataView.h"
#include "RealTreeView.h"
#include "RealListView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRealDataView

IMPLEMENT_DYNCREATE(CRealDataView, CView)

BEGIN_MESSAGE_MAP(CRealDataView, CView)
	//{{AFX_MSG_MAP(CRealDataView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRealDataView construction/destruction

CRealDataView::CRealDataView()
{
	// TODO: add construction code here

}

CRealDataView::~CRealDataView()
{
}

BOOL CRealDataView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRealDataView drawing

void CRealDataView::OnDraw(CDC* pDC)
{
	CRealDataDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CRealDataView printing

BOOL CRealDataView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CRealDataView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CRealDataView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CRealDataView diagnostics

#ifdef _DEBUG
void CRealDataView::AssertValid() const
{
	CView::AssertValid();
}

void CRealDataView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CRealDataDoc* CRealDataView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRealDataDoc)));
	return (CRealDataDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRealDataView message handlers

int CRealDataView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
    //Create the main splitter view. There are two panes (2 columns, 1 row).
    //The left pane is a tree view, the right pane is itself a splitter view.
    m_wndSplitter.CreateStatic(this, 1, 2);
    CCreateContext *pContext = (CCreateContext*) lpCreateStruct->lpCreateParams;
    lpCreateStruct->style |= WS_OVERLAPPED;

    m_wndSplitter.CreateView(0, 0, RUNTIME_CLASS(CRealTreeView), CSize(0,0), pContext);
    m_wndSplitter.CreateView(0, 1, RUNTIME_CLASS(CRealListView), CSize(0,0), pContext);

    return 0; // OnCreate must return 0 to continue the creation of the CWnd object
}

void CRealDataView::OnInitialUpdate() 
{
    CView::OnInitialUpdate();
	
//    ((CView*)m_wndSplitter.GetPane(0, 0))->OnInitialUpdate();
//    ((CView*)m_wndSplitter.GetPane(0, 1))->OnInitialUpdate();
}

void CRealDataView::OnSize(UINT nType, int cx, int cy) 
{
    CView::OnSize(nType, cx, cy);

    m_wndSplitter.MoveWindow(0, 0, cx, cy);

    //We just want to set the X column upon creation of the view. This way the user can  
    //move the splitter bar to how they like it and still resize the frame window 
    //without it snapping back:
    //if(m_bShouldSetXColumn) 
    m_wndSplitter.SetColumnInfo(0, cx/6, 0);

    m_wndSplitter.RecalcLayout();  
}

CRealTreeView *CRealDataView::GetTreeView()
{
    return (CRealTreeView *) m_wndSplitter.GetPane(0, 0);
}

CRealListView *CRealDataView::GetListView()
{
    return (CRealListView *) m_wndSplitter.GetPane(0, 1);
}


