/**************************************************************************
**
** Copyright (C) Computer Science Department, Worcester Polytechnic Institute,
**                           All rights reserved.
**
** FILE NAME: RealTraceData.cpp
**
** DESCRIPTION: This program implements the methods of CRealTraceData class
**              
**
** AUTHOR: Zheng Zuo/Yubing Wang        March-2001
**
/**************************************************************************/



// RealTraceData.cpp: implementation of the CRealTraceData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RealData.h"
#include "RealTraceData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRealTraceData::CRealTraceData()
{
    m_nItemNum = 0;
    m_nMaxItem = 32;
    m_szItemName = new CString [m_nMaxItem];
    m_bItemRead = new BOOL [m_nMaxItem];
    m_nSubItemNum = new int [m_nMaxItem];
    m_szFilePath = new CStringArray [m_nMaxItem];
    m_pRealData = new CPtrArray [m_nMaxItem];
}

CRealTraceData::~CRealTraceData()
{
    if (m_szItemName != NULL)
        delete [] m_szItemName;
    if (m_bItemRead != NULL)
        delete [] m_bItemRead;
    if (m_nSubItemNum != NULL)
        delete [] m_nSubItemNum;
    if (m_szFilePath != NULL)
        delete [] m_szFilePath;
    if (m_pRealData != NULL)
    {
        int i, j;
        for (i=0; i<m_nItemNum; i++)
        {
            for (j=0; j<m_pRealData[i].GetSize(); j++)
                delete (CRealData *) m_pRealData[i][j];
        }
        delete [] m_pRealData;
    }
}

int CRealTraceData::GetItemNum()
{
    if (m_nItemNum == 0 || m_szItemName == NULL)
        ReadRootDir();
    return m_nItemNum;
}

CString CRealTraceData::GetItemName(int index)
{
    if (m_nItemNum == 0 || m_szItemName == NULL)
        ReadRootDir();
    return m_szItemName[index];
}

int CRealTraceData::GetSubItemNum(int index)
{
    if (m_nItemNum == 0 || m_szItemName == NULL)
        ReadRootDir();
    if (! m_bItemRead[index])
        ReadSubDir(index);
    return m_nSubItemNum[index];
}

CString CRealTraceData::GetSubItemPath(int index, int subIndex)
{
    return m_szFilePath[index][subIndex];
}

CRealData *CRealTraceData::GetSubItemData(int index, int subIndex)
{
/*    if (m_nItemNum == 0 || m_szItemName == NULL)
        ReadRootDir();
    if (! m_bItemRead[index])
        ReadSubDir(index);*/
    if (! m_bItemRead[index])
    {
        CRealData *data;
        int num = m_nSubItemNum[index];
        for (int i=0; i<num; i++)
        {
            data = new CRealData;
            data->FileName = m_szFilePath[index][i];
            ReadTracerFile(m_szFilePath[index][i], *data);
            m_pRealData[index].Add(data);
        }

        m_bItemRead[index] = TRUE;
    }
    return (CRealData *) m_pRealData[index][subIndex];
}

void CRealTraceData::AddItem(CString szItemName)
{
    if (m_nItemNum == m_nMaxItem)
    {
        m_nMaxItem += 16;

        int i;

        CString *oldItemName = m_szItemName;
        m_szItemName = new CString [m_nMaxItem];
        for (i=0; i<m_nItemNum; i++)
            m_szItemName[i] = oldItemName[i];
        delete [] oldItemName;

        BOOL *oldbItemRead = m_bItemRead;
        m_bItemRead = new BOOL [m_nMaxItem];
        for (i=0; i<m_nItemNum; i++)
            m_bItemRead[i] = oldbItemRead[i];
        delete [] oldbItemRead;

        int *oldSubItemNum = m_nSubItemNum;
        m_nSubItemNum = new int [m_nMaxItem];
        for (i=0; i<m_nItemNum; i++)
            m_nSubItemNum[i] = oldSubItemNum[i];
        delete [] oldSubItemNum;

        CStringArray *oldFilePath = m_szFilePath;
        m_szFilePath = new CStringArray [m_nMaxItem];
        for (i=0; i<m_nItemNum; i++)
            m_szFilePath[i].Copy(oldFilePath[i]);
        delete [] oldFilePath;

        CPtrArray *oldpRealData = m_pRealData;
        m_pRealData = new CPtrArray [m_nMaxItem];
        for (i=0; i<m_nItemNum; i++)
            m_pRealData[i].Copy(oldpRealData[i]);
        delete [] oldpRealData;
    }

    m_szItemName[m_nItemNum] = szItemName;
    m_bItemRead[m_nItemNum] = FALSE;
    m_nSubItemNum[m_nItemNum] = 0;
    m_szFilePath[m_nItemNum].SetSize(0, 16);
    m_pRealData[m_nItemNum].SetSize(0, 16);
    m_nItemNum ++;
}

void CRealTraceData::AddSubItem(int index, CString filePath)
{
    m_nSubItemNum[index] ++;
    m_szFilePath[index].Add(filePath);
    if (m_bItemRead[index])
    {
        CRealData *data = new CRealData;
        data->FileName = filePath;
        ReadTracerFile(filePath, *data);
        m_pRealData[index].Add(data);
    }
}

void CRealTraceData::DeleteSubItem(int index, int subIndex)
{
    ASSERT(subIndex < m_nSubItemNum[index]);
    m_szFilePath[index].RemoveAt(subIndex);
    if (m_bItemRead[index])
    {
        delete (CRealData *)m_pRealData[index][subIndex];
        m_pRealData[index].RemoveAt(subIndex);
    }
    m_nSubItemNum[index] --;
}

// Read and get all file name "RealRootDir"(RealTracer), store in the variable 
// m_nItemNum & m_szItemName
void CRealTraceData::ReadRootDir()
{
/*	HANDLE hSearch;

	WIN32_FIND_DATA FindFileData;
 			
	CString FirstDir = m_szRootDir + "\\*.*";
  
	BOOL fFinished = FALSE; 

	m_nItemNum = 0; 

    m_szItemName = new CString [100];
 
	hSearch = FindFirstFile(FirstDir, &FindFileData);
	if (hSearch == INVALID_HANDLE_VALUE) 
	{ 
		MessageBox(NULL, "No more files", 
                "Search completed.", MB_OK);
	} 

    if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0)
	    m_szItemName[m_nItemNum ++] = FindFileData.cFileName; 

	while (!fFinished) 
	{ 
 
		if (!FindNextFile(hSearch, &FindFileData)) 
		{
			if (GetLastError() == ERROR_NO_MORE_FILES) 
			{ 
				fFinished = TRUE; 
			} 
			else 
			{ 
			   MessageBox(NULL, "could not find next file", 
                "Error.", MB_OK); 
			} 
		}
		else if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0)
		{
			m_szItemName[m_nItemNum ++] = FindFileData.cFileName; 
		}
	} 
 
	// Close the search handle. 
 
	if (!FindClose(hSearch)) 
	{ 
		MessageBox(NULL, "Could not closeSearch Handle", 
                "Error.", MB_OK);
	} 	

*/
}

// Read one directory, get all log file information
void CRealTraceData::ReadSubDir(int index)
{
/*    CString dirname = ((m_szRootDir + "\\") + m_szItemName[index]) + "\\";
	
	HANDLE hSearch;

	WIN32_FIND_DATA FindFileData;
 			
	CString FirstDir = dirname + "*.*";
  
	BOOL fFinished = FALSE; 

	int num = 0; 

    // Get the number of this directory
	hSearch = FindFirstFile(FirstDir, &FindFileData);
    while (hSearch)
    {
        if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0)
            num ++;

		if (! FindNextFile(hSearch, &FindFileData))
            break;
	} 
 
	// Close the search handle. 
	if (!FindClose(hSearch)) 
	{ 
		MessageBox(NULL, "Could not closeSearch Handle", 
                "Error.", MB_OK);
	} 	

    m_pRealData[index] = new CRealData [num];
    num = 0;
 
	hSearch = FindFirstFile(FirstDir, &FindFileData);
	if (hSearch == INVALID_HANDLE_VALUE) 
	{ 
		MessageBox(NULL, "No more files", 
                "Search completed.", MB_OK);
	} 

    if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0)
    {
	    m_pRealData[index][num].FileName = FindFileData.cFileName;
        ReadTracerFile(dirname + m_pRealData[index][num].FileName, m_pRealData[index][num]);
        num ++;
    }

	while (!fFinished) 
	{ 
 
		if (!FindNextFile(hSearch, &FindFileData)) 
		{
			if (GetLastError() == ERROR_NO_MORE_FILES) 
			{ 
				fFinished = TRUE; 
			} 
			else 
			{ 
			   MessageBox(NULL, "could not find next file", 
                "Error.", MB_OK); 
			} 
		}
		else if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0)
		{
	        m_pRealData[index][num].FileName = FindFileData.cFileName; 
            ReadTracerFile(dirname + m_pRealData[index][num].FileName, m_pRealData[index][num]);
            num ++;
		}
	} 
 
	// Close the search handle. 
 
	if (!FindClose(hSearch)) 
	{ 
		MessageBox(NULL, "Could not closeSearch Handle", 
                "Error.", MB_OK);
	} 	
	

    m_bItemRead[index] = TRUE;
    m_nSubItemNum[index] = num;

*/}

// Read one log file, store all information to structure data
void CRealTraceData::ReadTracerFile(CString filename, CRealData &data)
{
	FILE *fp;  

	char buf[256];
	CString line;
	double (*Real_Data)[20];
	CString Country;
	CString State;
	int pos;
	int index=0;
	int i;
	double FR_Sum;
    char *ptr;
	int frStart = 0; 
    int hh, mm, ss;

	if ((fp = fopen(filename, "r")) == NULL)
	    MessageBox(NULL, "Could not open file", "Error.", MB_OK);

    data.Quality = -1;
	data.AvgJitter = 0.0;
	data.VideoIndex = -1;
	

	while ( fgets(buf, 400, fp) )
	{
          if (strlen(buf) > 0 && buf[strlen(buf)-1] == 10)
	  	      buf[strlen(buf)-1] = 0;
		  line = buf;

		  if ((pos = line.Find("User Location (Country):")) != -1)
		  {
			 Country = line.Mid(pos+25);
		  }
		  
		  else if ((pos = line.Find("User Location (State/Province):")) != -1)
		  {
			  State = line.Mid(pos+32);
		  }
		  else if ((pos = line.Find("IP Address:")) != -1)
		  {
			  data.IPAddress  = line.Mid(pos+12);
		  }
		  else if ((pos = line.Find("CPU (user-defined):")) != -1)
		  {
			  data.CPUType = atoi(line.Mid(pos+20));
		  }
		  
		  else if ((pos = line.Find("Memory(M):")) != -1)
		  {
			  data.Memory = atoi(line.Mid(pos+11));
		  }
		  else if ((pos = line.Find("OS Type:")) != -1)
		  {
			  data.OSType = atoi(line.Mid(pos+9));
		  }
		  else if ((pos = line.Find("Network Configuration:")) != -1)
		  {
			  data.NetConfig = atoi(line.Mid(pos+23));
		  }
		  
		  else if ((pos = line.Find("Start time:")) != -1)
		  {
			  //data.Time = atoi(line.Mid(pos+12));
              pos += (12 + 11);
              hh = (line[pos] - '0') * 10 + (line[pos+1] - '0');
              mm = (line[pos+3] - '0') * 10 + (line[pos+4] - '0');
              ss = (line[pos+6] - '0') * 10 + (line[pos+7] - '0');
              data.Time = hh * 3600 + mm * 60 + ss;
		  }
		  
		  else if ((pos=line.Find("rtsp")) != -1)
		  {
			  data.VideoName = line.Left(pos-2); 
			  data.VideoName = data.VideoName.Mid(11); 
		  }
		  
		  else if ((pos=line.Find("pnm")) != -1)
		  {
			  data.VideoName = line.Left(pos-2);
			  data.VideoName = data.VideoName.Mid(11); 
		  }

		  else if ((pos = line.Find("Server:")) != -1)
		  {
			  data.RealServerInfo = line.Mid(pos+8);
		  }

		  else if ((pos = line.Find("Codec:")) != -1)
		  {
			  data.RealCodec = line.Mid(pos+7);
		  }

		  else if ((pos = line.Find("Quality:")) != -1)
		  {
              if (isdigit(line[pos+9]))
			    data.Quality = atoi(buf+9);
		  }

		  else if ((pos = line.Find("CodedBw AvgBw")) != -1)
		  {
              int CurPosition = ftell(fp);
              int data_num = 0;
              while ( fgets(buf, 400, fp) )
	          {
                      if (strlen(buf) > 0 && buf[strlen(buf)-1] == 10)
	  	                  buf[strlen(buf)-1] = 0;
		              line = buf;
                      if ((pos = line.Find("CodedBw AvgBw")) != -1)
                          break;
                      data_num ++;
              }
              fseek(fp, CurPosition, SEEK_SET);

              Real_Data = new double [data_num][20];
              while ( fgets(buf, 400, fp) )
	          {
                      if (strlen(buf) > 0 && buf[strlen(buf)-1] == 10)
	  	                  buf[strlen(buf)-1] = 0;
		              line = buf;
                      if ((pos = line.Find("CodedBw AvgBw")) != -1)
                          break;

                      ptr = strstr(buf, "#J");
                      while (ptr != NULL)
                      {
                          ptr[0] = ptr[1] = ' ';
                          ptr = strstr(ptr+2, "#J");
                      }

                      sscanf(buf, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
                                                                   &Real_Data[index][0],&Real_Data[index][1],
														           &Real_Data[index][2],&Real_Data[index][3],
														           &Real_Data[index][4],&Real_Data[index][5],
														           &Real_Data[index][6],&Real_Data[index][7],
														           &Real_Data[index][8],&Real_Data[index][9],
														           &Real_Data[index][10],&Real_Data[index][11],
														           &Real_Data[index][12],&Real_Data[index][13],
														           &Real_Data[index][14]);
			          index++;
              }
          }
    }


	data.CodedFR = index > 0 ? (int) Real_Data[index-1][6] : 0;

	data.CodedBW = index > 0 ? (int) Real_Data[index-1][0] : 0;

	data.AvgJitter = index > 0 ? Real_Data[index-1][11] : 0.0;

	data.TransProtocol = index > 0 ? (int)Real_Data[index-1][5] : 0;
		  
    data.Location = Country + "/" + State;

    FR_Sum = 0.0; 
    for (i=0; i < index; i++)
        FR_Sum += Real_Data[i][1];

    data.AvgBandWidth = index > 0 ? FR_Sum/index : 0.0;

    FR_Sum = 0.0; 
    for (i=0; i < index; i++)
	{
		if(Real_Data[i][8] > 0)
		{
			frStart = i;
			break;
		}
	}

	for (i=frStart; i < index; i++)
        FR_Sum += Real_Data[i][8];

    data.AvgFrameRate = index > 0 ? FR_Sum/index : 0.0;

    data.FrameDropped = index > 0 ? ((int) Real_Data[index-1][9]) : 0;
    
	data.FrameLost = index > 0 ? ((int) Real_Data[index-1][10]) : 0;

    FR_Sum = 0.0; 
    for (i=0; i < index; i++)
        FR_Sum += Real_Data[i][14];

    data.AvgCpuUsage = index > 0 ? FR_Sum/index : 0.0;
    fclose (fp);

    if (index > 0 && Real_Data != NULL)
        delete [] Real_Data;

    data.DataNum = index;
}

